import pandas as pd
import matplotlib
import matplotlib.pyplot as plt
import string
import random

import pandana
from pandana.loaders import osm

import geopy
from geopy.geocoders import Nominatim
from geopy.extra.rate_limiter import RateLimiter
from shapely.geometry import box

import warnings
warnings.filterwarnings('ignore')

def initialize_OSM_network(aoi_geojson)->pandana.network.Network:
    """
    takes in a geojson and outputs an OSM network preprocessed by Pandana.
    """
    
    # Set AOI CSV from geojson
    aoi_csv = aoi_geojson[0][0][1], aoi_geojson[0][3][0], aoi_geojson[0][2][1], aoi_geojson[0][1][0]

    # Get network from geocsv
    network = osm.pdna_network_from_bbox(*aoi_csv)
    
    return network

def node_query(aoi_csv, amenity):
    """
    Perform OSM's node_query function with try catch, using input geocsv and the specific amenity to query.
    """
    
    try:
        query = osm.node_query(*aoi_csv, tags=f'"amenity"="{amenity}"')
        return query
    except:
        pass

def get_OSM_network_data(network, df, aoi_geojson, poi_amenities, num_pois, maxdist, show_viz=False):
    """
    Input
        network: Pandana network
        df: a dataframe of longitude and latitude
        aoi_gejson: geojson of bounding box
        poi_amenities: List of amenities
                     : Choose available amenities in this documentation: https://wiki.openstreetmap.org/wiki/Map_features#Amenity
        num_pois: integer, number of points of interest to map and perform contraction hierarchies
        maxdist: in meters, maximum distance to perform contraction hierarchies
        show_viz: boolean, shows viz of map containing information from contraction hierarchy operations
    Returns
        final_df: same dataframe with concatenated data from points of interest
        amenities_df: dataframe on amenities from POIs
        count_distance_df: dataframe of counts and distances of input df longlat to amenities POIs
    """
    
    # Set AOI CSV from geojson
    aoi_csv = aoi_geojson[0][0][1], aoi_geojson[0][3][0], aoi_geojson[0][2][1], aoi_geojson[0][1][0]
    
    # get network ID per longlat pair of sampled points
    df['OSM_network_id'] = network.get_node_ids(df['longitude'], df['latitude'])

    # Set category string
    category = f'all_{"_".join(poi_amenities)}'

    # query node details for each ammenity
    amenities_dict = {poi_amenities[i]:node_query(aoi_csv, poi_amenities[i]) for i in range(len(poi_amenities))}
#     amenities_dict = {poi_amenities[i]:osm.node_query(*aoi_csv, tags=f'"amenity"="{poi_amenities[i]}"') for i in range(len(poi_amenities))}

    # Combine list of POIs into dataframe
    amenities_df = pd.concat(list(amenities_dict.values()))
    amenities_df = amenities_df[['lat', 'lon', 'amenity', 'name',]+[i for i in amenities_df.columns if 'addr' in i]]

    # Set POIs in network
    network.set_pois(category = category,
                         maxdist = maxdist,
                         maxitems = num_pois,
                         x_col = amenities_df.lon, 
                         y_col = amenities_df.lat)

    # Calculate distances
    results = network.nearest_pois(distance = maxdist,
                                   category = category,
                                   num_pois = num_pois,
                                  # include_poi_ids = True
                                  )

    # Get distance of n nearest POIs
    distance_df = results.reset_index().rename(columns={'id':'OSM_network_id'}).merge(df[['OSM_network_id']])
    distance_df.columns = ['OSM_network_id'] + [f'nearest_{"_".join(poi_amenities)}_{i}' for i in range(1, num_pois+1)]

    # Count of healthcare establishments around each node
    amenities_nodes = network.get_node_ids(amenities_df.lon, amenities_df.lat)

    # Set amenities nodes on the network
    network.set(amenities_nodes, 
                name = category)

    # Count accessibility score for number of POIs within distance of each node
    accessibility = network.aggregate(distance = maxdist,
                                      type = 'count',
                                      name = category)

    # Count number of nearest POIs
    count_df = accessibility.reset_index().rename(columns={'id':'OSM_network_id'}).merge(df[['OSM_network_id']])
    count_df.columns = ['OSM_network_id', f'count_{"_".join(poi_amenities)}_within_{maxdist/1000.}km']

    # Merge count and distance
    count_distance_df = distance_df.merge(count_df)

    # Merge with final_df 
    final_df = df.merge(count_distance_df)

    if show_viz==True:
        fig, ax = plt.subplots(figsize=(10,8))

        plt.title(f'Distribution of {"_".join(poi_amenities)} Points of Interest ({maxdist/1000.}km radius)')
        plt.scatter(network.nodes_df.x, network.nodes_df.y, 
                    c=accessibility, s=1, cmap='Blues', 
                    norm=matplotlib.colors.LogNorm())
        cb = plt.colorbar()
        plt.show()
        
    return final_df, amenities_df, count_distance_df

def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))

def reverse_geocode(lat, long, user_agent_string='geogeopypy')->pd.DataFrame:
    """
    Takes in latlong and outputs a dataframe containing geocode details.
    """
    
    locator = Nominatim(user_agent=user_agent_string)
    coordinates = f"{lat}, {long}"
    
    rgeocode = RateLimiter(locator.reverse, min_delay_seconds=0.001)
    loc_details = rgeocode(coordinates).raw
    loc_details_df = pd.json_normalize(loc_details)
    return loc_details_df

def reverse_geocode_points(df, latitude='latitude', longitude='longitude')->pd.DataFrame:
    """
    Takes in a dataframe with longitude and latitude, perfoms reverse geocode and outputs the same df
    with concatenated geocode information.
    """
    
    # set ID
    id_str = id_generator()
    
    # reverse geocode points 
    series = df[[latitude, longitude]].apply(lambda x: reverse_geocode(x[0], x[1], user_agent_string=id_str), axis=1)
    points_rgeocode_df = pd.concat(series.tolist())

    # concatenate to original df
    points_with_rgeo_df = pd.concat([df, points_rgeocode_df.reset_index()], axis=1)
    
    return points_with_rgeo_df

def reverse_geocode_center_of_geojson(aoi_geojson)->str:
    """
    Takes in a geojson and outputs an address.
    """
    
    # set ID
    id_str = id_generator()
    
    # Initialize geolocator
    geolocator = Nominatim(user_agent=id_str)
    
    # Set bounds as polygon
    bounds = aoi_geojson[0][0][1], aoi_geojson[0][3][0], aoi_geojson[0][2][1], aoi_geojson[0][1][0]
    polygon = box(*bounds)
    
    # reverse geocode aoi_csv 
    reverse_geocode = geolocator.reverse(", ".join([str(i) for i in [polygon.centroid.x, polygon.centroid.y]]))
    
    return reverse_geocode.address



import pandas as pd
import folium

from sklearn.cluster import KMeans as km
import ee

import geopandas as gpd

def authenticate():
    """
    Authenticate connection to the server
    Please see documentation for auth: https://developers.google.com/earth-engine/guides/python_install-conda#mac_6
    """
    
    ee.Authenticate()

def initialize():
    """
    Initialize connection to server
    """
    ee.Initialize()

def meanNDVICollection(img, aoi)->float:
    """
    NDVI = (NIR – Red) / (NIR + Red)
    Vegetation index for observe greenery.
    NDVI = -1 to 0 represent Water bodies
    NDVI = -0.1 to 0.1 represent Barren rocks, sand, or snow
    NDVI = 0.2 to 0.5 represent Shrubs and grasslands or senescing crops
    NDVI = 0.6 to 1.0 represent Dense vegetation or tropical rainforest
    """
    
    nir = img.select('SR_B5')
    red = img.select('SR_B4')
    
    ndviImage = nir.subtract(red).divide(nir.add(red)).rename('NDVI')
    
    # Compute the mean of NDVI over the 'region'
    ndviValue = ndviImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('NDVI');  

    return ndviValue.getInfo()

def meanNDBICollection(img, aoi)->float:
    """
    Index for measuring built-up areas (buildings ,etc).
    NDBI values range from -1 to 1.
    Negative value of NDBI represent water bodies where as higher value represent build-up areas 
   
    """
    
    b5 = img.select('SR_B5')
    b6 = img.select('SR_B6')
    
    ndbiImage = b6.subtract(b5).divide(b6.add(b5)).rename('NDBI')
    
    # Compute the mean of NDBI over the 'region'
    ndbiValue = ndbiImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('NDBI');  

    return ndbiValue.getInfo()

def meanNDWICollection(img, aoi)->float:
    """
    Normalize Difference Water Index (NDWI) is use for the water bodies analysis.
    MNDWI = (Green – SWIR) / (Green + SWIR)
    Landsat 8 data, NDWI = (Band 3 – Band 6) / (Band 3 + Band 6)
    Normalize Difference Water Index (NDWI) value lies between -1 to 1. 
    Generally, water bodies NDWI value is greater than 0.5.
    """
    
    b3 = img.select('SR_B3')
    b6 = img.select('SR_B6')
    
    ndwiImage = b3.subtract(b6).divide(b3.add(b6)).rename('NDWI')
    
    # Compute the mean of NDWI over the 'region'
    ndwiValue = ndwiImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('NDWI'); 

    return ndwiValue.getInfo()

def meanNDMICollection(img, aoi)->float:
    """
    Normalized Difference Moisture Index (NDMI) is used to determine vegetation water content. 
    It is calculated as a ratio between the NIR and SWIR values in traditional fashion.
    In Landsat 8, NDMI = (Band 5 – Band 6) / (Band 5 + Band 6).
    """
    
    b5 = img.select('SR_B5')
    b6 = img.select('SR_B6')
    
    ndmiImage = b5.subtract(b6).divide(b5.add(b6)).rename('NDMI')
    
    # Compute the mean of NDMI over the 'region'
    ndmiValue = ndmiImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('NDMI'); 
    
    return ndmiValue.getInfo()

def meanfAPARCollection(img, aoi)->float:
    """
    The MCD15A3H V6 level 4, Combined Fraction of Photosynthetically Active Radiation (FPAR), 
    and Leaf Area Index (LAI) product is a 4-day composite data set with 500 meter pixel size. 
    The algorithm chooses the "best" pixel available from all the acquisitions of both MODIS sensors located on
    NASA's Terra and Aqua satellites from within the 4-day period.
    
    Range: min value is 0, max value is 100 with scale factor of 0.01.
    """
    
    fapar = img.select('Fpar')
    
    faparImage = fapar.rename('fapar')
    
    # Compute the mean of of precipitation over the 'region'
    faparValue = faparImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('fapar'); 

    try:
        return faparValue.getInfo() * 0.001
    except:
        return 0

def meanAirQualityCollection(img, aoi)->float:
    """
    Aerosol Index is a qualitative index indicating the presence of elevated layers of aerosols 
    with significant absorption. The main aerosol types that cause signals detected in the AI are desert dust, 
    biomass burning and volcanic ash plumes. 
    An advantage of the AI is that it can be derived for clear as well as (partly) cloudy ground pixels.
    """
    
    aerosol = img.select('SR_QA_AEROSOL')
    
    aerosolImage = aerosol.rename('aerosol')
    
    # Compute the mean of aerosol (air quality index) over the 'region'
    aerosolValue = aerosolImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('aerosol'); 

    return aerosolValue.getInfo()

def meanSurfaceTemperatureCollection(img, aoi)->float:
    """
    Provides daily land surface temperature (LST) and emissivity values in a 1200 x 1200 kilometer grid.
    Digital numbers range from 7500 to 65535 with a scale factor of 0.02 (converts to Kelvin)
    """
    
    surftemp = img.select('LST_Day_1km')
    
    surftempImage = surftemp.rename('surface_temperature')
    
    surftempValue = surftempImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('surface_temperature');  # result of reduceRegion is always a dictionary, so get the element we want

    try:
        return surftempValue.getInfo() * 0.02 - 273.15 # converting LST Digital Number to Deg Celsius
    except:
        return None


def meanPrecipitationCollection(img, aoi)->float:
    """
    Global Land Data Assimilation System (GLDAS) ingests satellite and ground-based observational data products. 
    Using advanced land surface modeling and data assimilation techniques, 
    it generates optimal fields of land surface states and fluxes.
    """
    
    precip = img.select('Rainf_f_tavg')
    
    precipImage = precip.rename('precipitation')
    
    # Compute the mean of of precipitation over the 'region'
    precipValue = precipImage.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('precipitation'); 

    return precipValue.getInfo()

def meanRelHumidityCollection(img, aoi)->float:
    """
    Global Land Data Assimilation System (GLDAS) ingests satellite and ground-based observational data products. 
    Using advanced land surface modeling and data assimilation techniques, 
    it generates optimal fields of land surface states and fluxes.
    Ranges from 0 to 100 (with estimation errors)
    """
    
    relative_humidity = img.expression(
      '0.263 * p * q * (exp(17.67 * (T - T0) / (T - 29.65))) ** -1', {
        'T': img.select('Tair_f_inst'),
        'T0': 273.16,
        'p': img.select('Psurf_f_inst'),
        'q': img.select('Qair_f_inst')
      }
    ).float().rename('relative_humidity')
    
    # Compute the mean of relative humidity over the 'region'
    relative_humidityValue = relative_humidity.reduceRegion(**{
    'geometry': aoi.getInfo(),
    'reducer': ee.Reducer.mean(),
    'scale': 1000
    }).get('relative_humidity'); 

    return relative_humidityValue.getInfo()

def get_satellite_measures_from_AOI(aoi_geojson, 
                           sample_points, 
                           landsat_catalog='LANDSAT/LC08/C02/T1_L2',
                           modis_catalog = "MODIS/006/MOD11A1",
                           gldas_catalog = "NASA/GLDAS/V021/NOAH/G025/T3H",
                           date_from='2021-11-01', 
                           date_to='2021-12-31')->pd.DataFrame:
    """
    From a bounding box geojson, get normalized difference indices at different sample points.
    """
    # Landsat catalog (for normalized difference indices)
    landsat = ee.ImageCollection(landsat_catalog)
    
    # MODIS catalog (for surface temperature)
    modis = ee.ImageCollection(modis_catalog)
    
    # GLDAS catalog (for precipitation)
    gldas = ee.ImageCollection(gldas_catalog)
    
    # MODIS catalog (for fAPAR)
    modis_fpar = ee.ImageCollection("MODIS/006/MCD15A3H")
    
    # setting the Area of Interest (AOI)
    AOI = ee.Geometry.Polygon(aoi_geojson)

    # filter area and date
    landsat_AOI = landsat.filterBounds(AOI).filterDate(date_from, date_to)
    
    # Get satellite image with lowest cloud cover
    sat_image = ee.Image(landsat_AOI.sort('CLOUD_COVER').first())
    
    # filter area and date for MODIS
    modis_AOI = modis.filterBounds(AOI).filterDate(date_from, date_to)

    # Get satellite image for MODIS
    modis_sat_image = ee.Image(modis_AOI.median())
    
    # filter area and date for GLDAS
    gldas_AOI = gldas.filterBounds(AOI).filterDate(date_from, date_to)

    # Get satellite image for GLDAS
    gldas_sat_image = ee.Image(gldas_AOI.median())
    
    # filter area and date for MODIS FAPAR
    modis_fpar_AOI = modis_fpar.filterBounds(AOI).filterDate(date_from, date_to)

    # Get satellite image for MODIS FAPAR
    modis_fpar_sat_image = ee.Image(modis_fpar_AOI.median())
    
    # Function to get 1km patches of images from each point
    roi_with_buffer_fn = lambda geopoint: ee.Geometry.Point([geopoint.xy[0][0], geopoint.xy[1][0]]).buffer(1000)
    
    # Get more features of interest
    points = ee.FeatureCollection.randomPoints(AOI, sample_points)
    
    # Convert ee.geometry points to pandas dataframe and add 1km buffer around each point
    points_df = gpd.GeoDataFrame.from_features(points.getInfo()["features"])
    points_df['buffered_geometry'] = points_df['geometry'].apply(roi_with_buffer_fn)

    # Extract long lat
    points_df['longitude'] = points_df.geometry.apply(lambda g: g.x)
    points_df['latitude'] = points_df.geometry.apply(lambda g: g.y)

    # Get all normalized difference indices
    points_df['ndvi'] = points_df['buffered_geometry'].apply(lambda x: meanNDVICollection(sat_image, x))
    points_df['fapar'] = points_df['buffered_geometry'].apply(lambda x: meanfAPARCollection(modis_fpar_sat_image, x))
    points_df['ndbi'] = points_df['buffered_geometry'].apply(lambda x: meanNDBICollection(sat_image, x))
    points_df['ndwi'] = points_df['buffered_geometry'].apply(lambda x: meanNDWICollection(sat_image, x))
    points_df['ndmi'] = points_df['buffered_geometry'].apply(lambda x: meanNDMICollection(sat_image, x))
    points_df['aerosol'] = points_df['buffered_geometry'].apply(lambda x: meanAirQualityCollection(sat_image, x))
    points_df['surface_temperature'] = (points_df['buffered_geometry']
                                        .apply(lambda x: meanSurfaceTemperatureCollection(modis_sat_image, x)))
    points_df['precipitation_rate'] = (points_df['buffered_geometry']
                                        .apply(lambda x: meanPrecipitationCollection(gldas_sat_image, x)))
    points_df['relative_humidity'] = (points_df['buffered_geometry']
                                        .apply(lambda x: meanRelHumidityCollection(gldas_sat_image, x)))
    
    return points_df

def perform_clustering(df, 
                       features=['longitude', 'latitude', 'ndvi', 'ndbi', 'ndwi', 'ndmi', 
                                 'surface_temperature', 'precipitation_rate', 'relative_humidity'],
                       n_clusters=5)->pd.Series:
    
    """
    From dataframe and preset list of features to cluster, output labels
    """
    
    X = df[features].dropna(axis=1, how='all')
    
    kmeans = km(n_clusters=n_clusters, 
                random_state=42).fit(X)
    
    labels_df = pd.Series(kmeans.labels_)
    
    return labels_df
    
def scale_factor(image):
  # scale factor for the MODIS MOD13Q1 product

  return image.multiply(0.0001).copyProperties(image, ['system:time_start'])


def get_time_series_ndvi_evi(geojson, date_from='2018-01-01', date_to='2021-12-31')->pd.DataFrame:
    """
    From geojson, start and end date, get time-seris NDVI and EVI
    """
    
    # Get AOI
    AOI = ee.Geometry.Polygon(geojson)
    
    # EVI and NDVI from MODIS NNASA
    catalog_to_use = 'MODIS/006/MOD13Q1' 
    
    # Define data range 
    date_range = ee.DateRange(date_from, date_to)
    
    # Get modis satellite image collection
    modis = ee.ImageCollection(catalog_to_use).filterDate(date_range)
    
    # select EVI, NDVI
    evi = modis.select('EVI')
    ndvi = modis.select('NDVI')
    
    # Mapping function to multiply by the scale factor
    scaled_evi = evi.map(scale_factor)
    scaled_ndvi = ndvi.map(scale_factor)
    
    # NDVI time series
    AOI_ndvi = chart.Image.series(**{'imageCollection': scaled_ndvi,
                                       'region': AOI,
                                       'reducer': ee.Reducer.mean(),
                                       'scale': 1000,
                                       'xProperty': 'system:time_start'})

    # EVI time series
    AOI_evi = chart.Image.series(**{'imageCollection': scaled_evi,
                                       'region': AOI,
                                       'reducer': ee.Reducer.mean(),
                                       'scale': 1000,
                                       'xProperty': 'system:time_start'})
    
    # concatenate NDVI and EVI indices
    vegetation_df = AOI_ndvi.dataframe
    vegetation_df['EVI'] = AOI_evi.dataframe['EVI']
    
    return vegetation_df

def visualize_on_map(points_df, ignore_labels=None, is_dark=True):
    """
    Visualize the clusters on the map using Folium
    Themese for TileLayer: https://deparkes.co.uk/2016/06/10/folium-map-tiles/
    """
    
    # Plot clusters
    viz_map = folium.Map(location=[points_df['latitude'].iloc[0],points_df['longitude'].iloc[0]], zoom_start=10)
    
    # Set to dark theme if toggled
    if is_dark:
        folium.TileLayer('cartodbdark_matter').add_to(viz_map)
    
    # if ignore_labels has input, remove them from unique labels
    if ignore_labels==None:
        unique_labels = points_df['labels'].unique().tolist()
    else:
        unique_labels = points_df['labels'].unique().tolist()
        unique_labels = [label for label in unique_labels if label not in ignore_labels]
    
    # set colors
    colors = ['lightred', 'orange', 'red', 'darkred', 'lightblue', 'blue', 'darkblue',
              'green', 'purple',  'beige',  'darkgreen', 'cadetblue', 
              'darkpurple', 'white', 'pink', 'lightgreen', 
              'gray', 'black', 'lightgray']
    
    for j in range(len(unique_labels)):
        for i in points_df[points_df['labels']==unique_labels[j]].index:
            folium.Marker(
            location = [points_df.geometry.iloc[i].y, points_df.geometry.iloc[i].x],
            popup = points_df.iloc[i]['labels'],
            icon = folium.Icon(color=colors[j])
            ).add_to(viz_map)

    return viz_map



import pandas as pd

import streamlit as st
import re

import aedes
from aedes.remote_sensing_utils import get_satellite_measures_from_AOI
from aedes.remote_sensing_utils import perform_clustering, visualize_on_map

from aedes.osm_utils import initialize_OSM_network, get_OSM_network_data, reverse_geocode_points, reverse_geocode_center_of_geojson

from streamlit_folium import folium_static

aedes.remote_sensing_utils.initialize()

st.title('AEDES: Predictive Geospatial Hostpot Detection')
st.write("""This web application demonstrates the use of satellite, weather and OpenStreetMap data to identify potential hotspots for vector-borne diseases. This web application only needs geojson input of an area of interest and then it automatically collects and models the data needed for hotspot detection at a longlat level.""")

st.subheader('Input Bounding Box Coordinates')

aoi_str = st.text_area("Input geojson of area of interest here", 
                             value=[[[120.98976275,14.58936896],
                                       [121.13383232,14.58936896],
                                       [121.13383232,14.77641364],
                                       [120.98976275,14.77641364],
                                       [120.98976275,14.58936896]]])

list_parsed = [float(re.findall("\d+\.\d+", num)[0]) for num in aoi_str.split(",")]

aoi_geojson = [[[list_parsed[0], list_parsed[1]],
              [list_parsed[2], list_parsed[3]],
              [list_parsed[4], list_parsed[5]],
              [list_parsed[6], list_parsed[7]],
              [list_parsed[8], list_parsed[9]]]]

st.subheader('Bounding Box Center')

st.write(f"Detect hotspots around {reverse_geocode_center_of_geojson(aoi_geojson)}...")

satellite_df = get_satellite_measures_from_AOI(aoi_geojson, 50)

satellite_df['labels'] = perform_clustering(satellite_df, n_clusters=3)

rev_geocode_df = reverse_geocode_points(satellite_df)

st.subheader('Detected Hotspots')

mapper = visualize_on_map(satellite_df, ignore_labels=[1])

folium_static(mapper)

st.subheader('Risky Locations')

indentified_risky_places_df = rev_geocode_df[rev_geocode_df['labels'].isin([0, 2])]

try:
    risk_df = indentified_risky_places_df[[i for i in rev_geocode_df.columns if 'address' in i]].fillna('').value_counts().reset_index().drop(0, axis=1)
    st.dataframe(risk_df)
except:
    st.write('OpenStreetMap returned no available data for each longitude-latitude pair.')
    
st.subheader('Top Villages at Risk')

try:
    risk_village_df = indentified_risky_places_df['address.village'].value_counts().reset_index().drop('address.village', axis=1).rename(columns={'index':'top_villages'})
    st.dataframe(risk_village_df)
except:
    st.write('OpenStreetMap returned no available data for suburbs.')  
    
st.subheader('Top Suburbs at Risk')

try:
    risk_suburb_df = indentified_risky_places_df['address.suburb'].value_counts().reset_index().drop('address.suburb', axis=1).rename(columns={'index':'top_suburbs'})
    st.dataframe(risk_suburb_df)
except:
    st.write('OpenStreetMap returned no available data for suburbs.')    

st.subheader('Top Cities at Risk')

try:
    risk_cities_df = indentified_risky_places_df['address.city'].value_counts().reset_index().drop('address.city', axis=1).rename(columns={'index':'top_cities'})
    st.dataframe(risk_cities_df)
except:
    st.write('OpenStreetMap returned no available data for cities.')

st.subheader('Top Postcodes at Risk')

try:
    risk_postcode_df = indentified_risky_places_df['address.postcode'].value_counts().reset_index().drop('address.postcode', axis=1).rename(columns={'index':'top_postcodes'})
    st.dataframe(risk_postcode_df)
except:
    st.write('OpenStreetMap returned no available data for postcodes.')

st.subheader('Top Regions at Risk')

try:
    risk_region_df = indentified_risky_places_df['address.region'].value_counts().reset_index().drop('address.region', axis=1).rename(columns={'index':'top_regions'})
    st.dataframe(risk_region_df)
except:
    st.write('OpenStreetMap returned no available data for region.')
